//=========================================================
// PSoC5LP Project
//=========================================================
// File Name : main.c
// Function  : Main Routine
//---------------------------------------------------------
// Rev.01 2012.08.14 Munetomo Maruyama
//---------------------------------------------------------
// Copyright (C) 2012-2013 Munetomo Maruyama
//=========================================================
// ---- License Information -------------------------------
// Anyone can FREELY use this code fully or partially
// under conditions shown below.
// 1. You may use this code only for individual purpose,
//    and educational purpose.
//    Do not use this code for business even if partially.
// 2. You can copy, modify and distribute this code.
// 3. You should remain this header text in your codes
//   including Copyright credit and License Information.
// 4. Your codes should inherit this license information.
//=========================================================
// ---- Patent Notice -------------------------------------
// I have not cared whether this system (hw + sw) causes
// infringement on the patent, copyright, trademark,
// or trade secret rights of others. You have all
// responsibilities for determining if your designs
// and products infringe on the intellectual property
// rights of others, when you use technical information
// included in this system for your business.
//=========================================================
// ---- Disclaimers ---------------------------------------
// The function and reliability of this system are not
// guaranteed. They may cause any damages to loss of
// properties, data, money, profits, life, or business.
// By adopting this system even partially, you assume
// all responsibility for its use.
//=========================================================

//---------------------------
// PSoC5 Includes
//---------------------------
#include <device.h>
//---------------------------
// RTOS Includes
//---------------------------
#include "FreeRTOS.h"
#include "queue.h"
#include "semphr.h"
#include "task.h"
//---------------------------
// API Includes
//---------------------------
#include "key.h"
#include "oled.h"
//-------------------------
// Define Radio Stations
//-------------------------
#define FBUSCLK 61333
#define FSAMPLE    25
#define TSAMPLE (FBUSCLK/(FSAMPLE*2))
#define STATION_MAX 4
char  *STATION_NAME[STATION_MAX] = {"NHK1", "NHK2", "AFN ", "TBS "};
uint32 STATION_FREQ[STATION_MAX] = {  594,    693,   810,    954}; // frequency of radio station
uint32 STATION_TLOC[STATION_MAX] = {   26,     22,    19,     16}; // period of counter for local osc
uint32 STATION_VCAP[STATION_MAX] = {   62,     90,   112,    136}; // varicap DAC level (for your reference)
//----------------------------------
// Define FreeRTOS related Resource
//----------------------------------
#define TASK_STACK_KEY   128
#define TASK_STACK_RADIO 512
#define TASK_PRIORITY_KEY   (tskIDLE_PRIORITY + 2)
#define TASK_PRIORITY_RADIO (tskIDLE_PRIORITY + 1)
//
void Task_KEY   (void *pvParameters);
void Task_RADIO (void *pvParameters);
//
xSemaphoreHandle xMutex_OLED;   // Mutex for OLED
xQueueHandle     xQueue_KEY[2]; // Queue Data from Task_Key_Push()
xQueueHandle     xQueue_IQ[2];  // Queue Data from CY_ISR(Filter_Handler)
//---------------------
// Define Prototype
//---------------------
void prvHardwareSetup(void);
void CORDIC(int32 x, int32 y, int32 *radius);
void Filter_Handler(void);
//-----------------------
// Define AGC Parameter
//-----------------------
#define AGC_TIM_MAX (25000 * 2) // 2sec
#define AGC_TIM_INI (AGC_TIM_MAX - 25000 / 4) // 1.75sec
//--------------------------
// State of Radio Operation
//--------------------------
enum STATE_RADIO {RADIO_NORMAL, RADIO_TRIM, RADIO_EEPROM};
//------------------
// Define Global
//------------------
uint32 gDO_TRIM_VCAP;

//=======================
// Main Routine
//=======================
void main()
{
    //---------------------------------------------
    // Set PO_POWER pin to continue power on state
    //---------------------------------------------
    PO_POWER_Write(1);    
    //--------------------------------------
    // Initialize Hardware related to RTOS
    //--------------------------------------
    prvHardwareSetup();
    CYGlobalIntEnable; // Enable Global Interrupts
    //-------------------
    // Initialize API
    //-------------------
    Init_Key();
    Init_OLED();
    //--------------------------
    // Welcome Message on OLED
    //--------------------------
    OLED_printf_Font(OLED_FONT_MEDIUM);
    OLED_printf_Position(0, 0);
    OLED_printf_Color(OLED_RED, OLED_BLK);
    OLED_printf("PSoC ");
    OLED_printf_Color(OLED_GRN, OLED_BLK);
    OLED_printf("5LP");
    //--------------------------
    // Determine if trim VCAP?
    //--------------------------
    gDO_TRIM_VCAP = Check_Key_Status(0); // push key0?
    while(Check_Key_Status(0)); // wait until key release
    //-------------------------------------
    // Create Task
    //-------------------------------------
    xMutex_OLED   = xSemaphoreCreateMutex();
    xQueue_KEY[0] = xQueueCreate(1, 1); // 1 x 1byte
    xQueue_KEY[1] = xQueueCreate(1, 1); // 1 x 1byte
    xQueue_IQ[0]  = xQueueCreate(8, 4); // 8 x 4bytes
    xQueue_IQ[1]  = xQueueCreate(8, 4); // 8 x 4bytes
    //
    xTaskCreate(Task_KEY,   (signed portCHAR *)"KEY",   TASK_STACK_KEY,   NULL, TASK_PRIORITY_KEY,   NULL);
    xTaskCreate(Task_RADIO, (signed portCHAR *)"RADIO", TASK_STACK_RADIO, NULL, TASK_PRIORITY_RADIO, NULL);
    //-------------------
    // Start RTOS Kernel 
    //-------------------
    vTaskStartScheduler();
    while(1);
}

//=======================================
// Hooks for Illegal Operations
//=======================================
void vApplicationStackOverflowHook(xTaskHandle pxTask, signed char *pcTaskName)
{
    // The stack space has been exceeded for a task, considering allocating more.
    taskDISABLE_INTERRUPTS();
    for( ;; );
}
void vApplicationMallocFailedHook(void)
{
    // The heap space has been exceeded.
    taskDISABLE_INTERRUPTS();
    for( ;; );
}

//======================================
// Hardware Setup
//======================================
void prvHardwareSetup(void)
{
    // Port layer functions that need to be copied into the vector table.
    extern void xPortPendSVHandler(void);
    extern void xPortSysTickHandler(void);
    extern void vPortSVCHandler(void);
    extern cyisraddress CyRamVectors[];
    
    // Install the OS Interrupt Handlers.
    CyRamVectors[11] = (cyisraddress) vPortSVCHandler;
    CyRamVectors[14] = (cyisraddress) xPortPendSVHandler;
    CyRamVectors[15] = (cyisraddress) xPortSysTickHandler;

    // Initialize Hardware
    //     later do it.
}

//========================
// CORDIC Routine (16bits)
// (calculate amplitude)
//========================
const int32 K = 39797; // 1/1.6467602 * 65536
void CORDIC(int32 x, int32 y, int32 *radius)
{
    int32 i;
    int32 u;
    //
    for (i = 0; i < 16; i++)
    {
        if (y >= 0) {u = x + (y >> i); y = y - (x >> i);}
        else        {u = x - (y >> i); y = y + (x >> i);}
        x = u;
    }
    //
    *radius = x; // x = xqrt(x^2 + y^2) * K  (K=1.6467602)
}

//============================
// Filter Interrupt Handler
//============================
CY_ISR(Filter_Handler)
{
    int32 filter_I;
    int32 filter_Q;

    if (Filter_1_IsInterruptChannelA())
    {
        filter_I = (int32) ((int16)Filter_1_Read16(Filter_1_CHANNEL_A));
        Filter_1_SR_REG = Filter_1_ALL_INTR & (~Filter_1_CHANNEL_B_INTR); // clear flags except INTR_HOLDB
        // Send the Data to Queue
        xQueueSendFromISR(xQueue_IQ[0], &filter_I, NULL);
    }
    if (Filter_1_IsInterruptChannelB())
    {
        filter_Q = (int32) ((int16)Filter_1_Read16(Filter_1_CHANNEL_B));
        Filter_1_SR_REG = Filter_1_ALL_INTR & (~Filter_1_CHANNEL_A_INTR); // clear flags except INTR_HOLDA
        // Send the Data to Queue
        xQueueSendFromISR(xQueue_IQ[1], &filter_Q, NULL);
    }
}

//==============================
// Task RADIO
//==============================
void Task_RADIO(void *pvParameters)
{
    uint32 state;
    uint32 i;
    uint8  *pEEPROM;
    uint8  EEPROM[16];
    int32  filter_I;
    int32  filter_Q;
    uint8  dac;
    int32  radius;
    uint32 station;
    uint32 station_update; 
    uint32 agc_tim;
    int32  agc_max;
    uint32 agc_mag;
    uint8  qmsg;
    //    
    // Variable declarations for DMA_1 and DMA_2
    uint8 DMA_1_Chan;
    uint8 DMA_1_TD[1];
    uint8 DMA_2_Chan;
    uint8 DMA_2_TD[1];
    //
    // DMA Configuration for DMA_1
    #define DMA_1_BYTES_PER_BURST   2 // keep my spoke until the bytes are transfered
    #define DMA_1_REQUEST_PER_BURST 1
    #define DMA_1_SRC_BASE (CYDEV_PERIPH_BASE)
    #define DMA_1_DST_BASE (CYDEV_PERIPH_BASE)
    DMA_1_Chan = DMA_1_DmaInitialize(DMA_1_BYTES_PER_BURST, DMA_1_REQUEST_PER_BURST, HI16(DMA_1_SRC_BASE), HI16(DMA_1_DST_BASE));
    DMA_1_TD[0] = CyDmaTdAllocate();
    CyDmaTdSetConfiguration(DMA_1_TD[0], 2, DMA_INVALID_TD, DMA_1__TD_TERMOUT_EN | TD_INC_SRC_ADR | TD_INC_DST_ADR);
    CyDmaTdSetAddress(DMA_1_TD[0], LO16((uint32)ADC_SAR_1_SAR_WRK0_PTR), LO16((uint32)Filter_1_STAGEA_PTR));
    CyDmaChSetInitialTd(DMA_1_Chan, DMA_1_TD[0]);
    CyDmaChEnable(DMA_1_Chan, 1); // restore TD configuration after a transaction
    //
    // DMA Configuration for DMA_2
    #define DMA_2_BYTES_PER_BURST   2 // keep my spoke until the bytes are transfered
    #define DMA_2_REQUEST_PER_BURST 1
    #define DMA_2_SRC_BASE (CYDEV_PERIPH_BASE)
    #define DMA_2_DST_BASE (CYDEV_PERIPH_BASE)
    DMA_2_Chan = DMA_2_DmaInitialize(DMA_2_BYTES_PER_BURST, DMA_2_REQUEST_PER_BURST, HI16(DMA_2_SRC_BASE), HI16(DMA_2_DST_BASE));
    DMA_2_TD[0] = CyDmaTdAllocate();
    CyDmaTdSetConfiguration(DMA_2_TD[0], 2, DMA_INVALID_TD, DMA_1__TD_TERMOUT_EN | TD_INC_SRC_ADR | TD_INC_DST_ADR);
    CyDmaTdSetAddress(DMA_2_TD[0], LO16((uint32)ADC_SAR_2_SAR_WRK0_PTR), LO16((uint32)Filter_1_STAGEB_PTR));
    CyDmaChSetInitialTd(DMA_2_Chan, DMA_2_TD[0]);
    CyDmaChEnable(DMA_2_Chan, 1); // restore TD configuration after a transaction
    //
    // Start EEPROM (Radio Station Memory)
    EEPROM_1_Start();
    pEEPROM = (uint8*) CYDEV_EE_BASE;
    for (station = 0; station < STATION_MAX; station++) STATION_VCAP[station] = pEEPROM[station];
    //
    // Initial State of Radio
    state = (gDO_TRIM_VCAP)? RADIO_TRIM : RADIO_NORMAL;
    //
    // Start Tunig Hardware
    VDAC8_2_Start();
    PGA_2_Start();
    //
    // Start Input PGA
    PGA_1_Start();
    //
    // Start Local IF
    Counter_1_Start();
    //
    // Start Mixers
    Mixer_1_Start();
    Mixer_2_Start();
    //
    // Start Audio DAC
    VDAC8_1_Start();
    //
    // Start Filters
    Filter_1_Start();
    Filter_1_INT_CTRL_REG = Filter_1_INT_CTRL_REG | 0x03; // to enable software polling
    //     STAGEA, STAGEB : L/M written by DMAC (key byte lane is M, need 8bit alignment)
    //     HOLDA,  HOLDB  : M/H read by CPU     (key byte lane is H, no need alignment)
    Filter_1_COHER_REG  = 0xa5;
    Filter_1_DALIGN_REG = 0x03;
    //
    // Enable Filter Interrupt
    isr_filter_StartEx(Filter_Handler);    
    //
    // Start ADC (ensure the origin of events should be started last)
    ADC_SAR_1_Start();
    ADC_SAR_2_Start();
    //
    // Start Sampling Signal
    Counter_2_Start();
    Counter_2_WritePeriod(TSAMPLE - 1);
    //
    // Default Radio Station
    station = 0;
    station_update = 1;
    //
    // Main Loop
    while(1)
    {   
        //
        // Switch Operation
        switch(state)
        {
            //-------------------------------
            // Normal Operation
            //-------------------------------
            case RADIO_NORMAL :
            {
                //
                // Update Display and Tuning
                if (station_update)
                {
                    station_update = 0;
                    OLED_Mutex_Begin();
                    OLED_printf_Font(OLED_FONT_SMALL);
                    OLED_printf_Position(0, 2);
                    OLED_printf_Color(OLED_YEL, OLED_BLK);
                    OLED_printf("SDR - AM Radio");
                    OLED_printf_Position(0, 4);
                    OLED_printf_Color(OLED_WHT, OLED_BLK);
                    OLED_printf("[%s]\n", STATION_NAME[station]);
                    OLED_printf("Freq=%3d\n", STATION_FREQ[station]);
                    OLED_printf("Tloc=%3d\n", STATION_TLOC[station]);
                    OLED_printf("Vcap=%3d\n", STATION_VCAP[station]);
                    OLED_Mutex_End();
                    //
                    // Set Tuning
                    Counter_1_WritePeriod(STATION_TLOC[station] - 1);
                    VDAC8_2_SetValue(STATION_VCAP[station]);
                    //
                    // Initialize AGC Control
                    agc_tim = AGC_TIM_INI;
                    agc_max = 0;
                    agc_mag = 0;
                }
                //
                // Change Selection of Station
                if (xQueueReceive(xQueue_KEY[0], &qmsg, 0) && (station < (STATION_MAX - 1)))
                {
                    station++;
                    station_update = 1;
                }
                if (xQueueReceive(xQueue_KEY[1], &qmsg, 0) && (station > 0))
                {
                    station--;
                    station_update = 1;
                }
               break;
            }
            //-------------------------------
            // Trimming VCAP
            //-------------------------------
            case RADIO_TRIM :
            {
                //
                // Update Display and Tuning
                if (station_update)
                {
                    station_update = 0;
                    OLED_Mutex_Begin();
                    OLED_printf_Font(OLED_FONT_SMALL);
                    OLED_printf_Position(0, 2);
                    OLED_printf_Color(OLED_YEL, OLED_BLK);
                    OLED_printf("SDR - Trim VCAP\n\n");
                    OLED_printf_Color(OLED_WHT, OLED_BLK);
                    OLED_printf("NAME FRQ LOC CAP");
                    OLED_printf("----------------");
                    for (i = 0; i <= station; i++)
                    {
                        OLED_printf_Position(0, i + 6);
                        OLED_printf("%s %3d %3d %3d", STATION_NAME[i], STATION_FREQ[i], STATION_TLOC[i], STATION_VCAP[i]);
                    }
                    OLED_Mutex_End();
                    //
                    // Set Tuning
                    Counter_1_WritePeriod(STATION_TLOC[station] - 1);
                    VDAC8_2_SetValue(STATION_VCAP[station]);
                    //
                    // Initialize AGC Control
                    agc_tim = AGC_TIM_MAX - 100;
                    agc_max = 0;
                    agc_mag = 0;
                }
                //
                // Check Keys
                if (xQueueReceive(xQueue_KEY[0], &qmsg, 0))
                {
                    STATION_VCAP[station] = (STATION_VCAP[station] + 1) & 0x00ff;
                    station_update = 1;
                }
                if (xQueueReceive(xQueue_KEY[1], &qmsg, 0))
                {
                    station++;
                    station_update = 1;
                    if (station >= STATION_MAX) state = RADIO_EEPROM;
                }
                break;
            }
            //-------------------------------
            // Write Trim Data to EEPROM
            //-------------------------------
            case RADIO_EEPROM :
            {
                if (station_update)
                {
                    station_update = 0;
                    OLED_Mutex_Begin();
                    OLED_printf_Font(OLED_FONT_SMALL);
                    OLED_printf_Color(OLED_CYN, OLED_BLK);
                    OLED_printf_Position(0, 15);
                    OLED_printf("Write EEPROM?");
                    OLED_Mutex_End();
                }
                //
                // OK?
                if (xQueueReceive(xQueue_KEY[1], &qmsg, 0))
                {
                    //
                    // Write EEPROM
                    for (i = 0; i < 16; i++)
                    {
                        EEPROM[i] = (i < STATION_MAX)? STATION_VCAP[i] : 0;
                    }
                    if (EEPROM_1_EraseSector(0) == CYRET_SUCCESS)
                    {
                        EEPROM_1_Write(EEPROM, 0);
                    }
                    pEEPROM = (uint8*) CYDEV_EE_BASE;
                    for (i = 0; i < STATION_MAX; i++) STATION_VCAP[i] = pEEPROM[i];
                    //
                    // Clear Screen
                    OLED_Mutex_Begin();
                    OLED_printf_Font(OLED_FONT_SMALL);
                    for (i = 2; i < 16; i++)
                    {
                        OLED_printf_Position(0, i);
                        OLED_printf("                ");
                    }
                    OLED_Mutex_End();
                    //
                    // Reset State
                    station = 0;
                    station_update = 1;
                    state = RADIO_NORMAL;
                }
                //
                // Cancel?
                if (xQueueReceive(xQueue_KEY[0], &qmsg, 0))
                {
                    //
                    // Clear Screen
                    OLED_Mutex_Begin();
                    OLED_printf_Font(OLED_FONT_SMALL);
                    for (i = 2; i < 16; i++)
                    {
                        OLED_printf_Position(0, i);
                        OLED_printf("                ");
                    }
                    OLED_Mutex_End();
                    //
                    // Reset State
                    station = 0;
                    station_update = 1;
                    state = RADIO_TRIM;
                }
                break;
            }
        }
        //----------------------------------------
        // Radio Data Processing
        //----------------------------------------
        //
        // Get Bandpass Result from Queue and Generate Audio Output
        if ((uxQueueMessagesWaiting(xQueue_IQ[0]) > 0) && (uxQueueMessagesWaiting(xQueue_IQ[1]) > 0))
        {
            xQueueReceive(xQueue_IQ[0], &filter_I, 0);        
            xQueueReceive(xQueue_IQ[1], &filter_Q, 0);        
            //      
            if (filter_I < 0) filter_I = -filter_I;
            if (filter_Q < 0) filter_Q = -filter_Q;
            //
            // Calculation of Amplitude
            CORDIC(filter_I, filter_Q, &radius);
            //
            // AGC
            if (agc_tim < AGC_TIM_MAX)
            {
                agc_max = (radius > agc_max)? radius : agc_max;
                agc_tim++;
            }
            else
            {
                agc_mag = (256 << 16) / agc_max; 
                agc_max = 0;
                agc_tim = 0;
            }
            radius = (radius * agc_mag) >> 16;
            //
            // Audio Output
            dac = (uint8) radius;
            VDAC8_1_SetValue(dac);
        }
    }
}

//=========================================================
// End of Program
//=========================================================
